#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2020 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/master/LICENSE.md

"""
Sensors Module

File : sensors.py

Brief : This file contains the definitions and functionalities for scanning a
        Redfish service's Power and Thermal properties for sensor readings
"""

def get_sensors( context ):
    """
    Walks a Redfish service for sensor information

    Args:
        context: The Redfish client object with an open session

    Returns:
        A list containing all sensor readings
    """

    sensor_list = []

    # Get the service root to find the chassis collection
    service_root = context.get( "/redfish/v1/" )
    if "Chassis" not in service_root.dict:
        # No chassis collection
        return sensor_list

    # Get the chassis collection and iterate through its collection
    chassis_col = context.get( service_root.dict["Chassis"]["@odata.id"] )
    for chassis_member in chassis_col.dict["Members"]:
        chassis = context.get( chassis_member["@odata.id"] )

        # Get the chassis status
        chassis_instance = {
            "ChassisName": chassis.dict["Id"],
            "Readings": []
        }
        sensor_list.append( chassis_instance )
        get_discrete_status( "State", chassis.dict, chassis_instance["Readings"] )

        # If the chassis contains any of the newer power/thermal models, scan based on the common sensor model
        if "EnvironmentMetrics" in chassis.dict or "PowerSubsystem" in chassis.dict or "ThermalSubsystem" in chassis.dict:
            # Get readings from the EnvironmentMetrics resource if available
            if "EnvironmentMetrics" in chassis.dict:
                environment = context.get( chassis.dict["EnvironmentMetrics"]["@odata.id"] )
                get_excerpt_status( chassis_instance["ChassisName"], "TemperatureCelsius", "Cel", environment.dict, chassis_instance["Readings"] )
                get_excerpt_status( chassis_instance["ChassisName"], "HumidityPercent", "%", environment.dict, chassis_instance["Readings"] )
                get_excerpt_status( chassis_instance["ChassisName"], "PowerWatts", "W", environment.dict, chassis_instance["Readings"] )
                get_excerpt_status( chassis_instance["ChassisName"], "EnergykWh", "kW.h", environment.dict, chassis_instance["Readings"] )
                get_excerpt_status( chassis_instance["ChassisName"], "PowerLoadPercent", "%", environment.dict, chassis_instance["Readings"] )
                get_excerpt_status( chassis_instance["ChassisName"], "DewPointCelsius", "Cel", environment.dict, chassis_instance["Readings"] )

            # Get readings from the PowerSubsystem resource if available
            if "PowerSubsystem" in chassis.dict:
                power = context.get( chassis.dict["PowerSubsystem"]["@odata.id"] )

                # Add information for each power supply reported
                if "PowerSupplies" in power.dict:
                    power_supplies = context.get( power.dict["PowerSupplies"]["@odata.id"] )
                    for power_supply_member in power_supplies.dict["Members"]:
                        power_supply = context.get( power_supply_member["@odata.id"] )
                        power_supply_name = "Power Supply " + power_supply.dict["Id"]
                        get_discrete_status( power_supply_name + " State", power_supply.dict, chassis_instance["Readings"] )
                        if "Metrics" in power_supply.dict:
                            metrics = context.get( power_supply.dict["Metrics"]["@odata.id"] )
                            get_excerpt_status( power_supply_name, "InputVoltage", "V", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( power_supply_name, "InputCurrentAmps", "A", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( power_supply_name, "InputPowerWatts", "W", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( power_supply_name, "EnergykWh", "kW.h", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( power_supply_name, "FrequencyHz", "Hz", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( power_supply_name, "OutputPowerWatts", "W", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( power_supply_name, "RailVoltage", "V", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( power_supply_name, "RailCurrentAmps", "A", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( power_supply_name, "RailPowerWatts", "W", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( power_supply_name, "TemperatureCelsius", "Cel", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( power_supply_name, "FanSpeedPercent", "%", metrics.dict, chassis_instance["Readings"] )

                # Add information for each battery reported
                if "Batteries" in power.dict:
                    batteries = context.get( power.dict["Batteries"]["@odata.id"] )
                    for battery_member in batteries.dict["Members"]:
                        battery = context.get( battery_member["@odata.id"] )
                        battery_name = "Battery " + battery.dict["Id"]
                        get_discrete_status( battery_name + " State", battery.dict, chassis_instance["Readings"] )
                        get_excerpt_status( battery_name, "StateOfHealthPercent", "%", battery.dict, chassis_instance["Readings"])
                        if "Metrics" in battery.dict:
                            metrics = context.get( battery.dict["Metrics"]["@odata.id"] )
                            get_excerpt_status( battery_name, "InputVoltage", "V", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( battery_name, "InputCurrentAmps", "A", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( battery_name, "OutputVoltages", "V", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( battery_name, "OutputCurrentAmps", "A", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( battery_name, "StoredEnergyWattHours", "W.h", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( battery_name, "StoredChargeAmpHours", "A.h", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( battery_name, "TemperatureCelsius", "Cel", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( battery_name, "ChargePercent", "%", metrics.dict, chassis_instance["Readings"] )
                            get_excerpt_status( battery_name, "CellVoltages", "V", metrics.dict, chassis_instance["Readings"] )

                # Add information for each of the redundancy groups reported
                if "PowerSupplyRedundancy" in power.dict:
                    for i, redundancy in enumerate( power.dict["PowerSupplyRedundancy"] ):
                        get_discrete_status( "Power Supply Redundancy " + str( i ), redundancy, chassis_instance["Readings"] )

            # Get readings from the ThermalSubsystem resource if available
            if "ThermalSubsystem" in chassis.dict:
                thermal = context.get( chassis.dict["ThermalSubsystem"]["@odata.id"] )

                # Add overall thermal metrics
                if "ThermalMetrics" in thermal.dict:
                    metrics = context.get( thermal.dict["ThermalMetrics"]["@odata.id"] )
                    if "TemperatureSummaryCelsius" in metrics.dict:
                        get_excerpt_status( chassis_instance["ChassisName"], "Internal", "Cel", metrics.dict["TemperatureSummaryCelsius"], chassis_instance["Readings"] )
                        get_excerpt_status( chassis_instance["ChassisName"], "Intake", "Cel", metrics.dict["TemperatureSummaryCelsius"], chassis_instance["Readings"] )
                        get_excerpt_status( chassis_instance["ChassisName"], "Exhaust", "Cel", metrics.dict["TemperatureSummaryCelsius"], chassis_instance["Readings"] )
                        get_excerpt_status( chassis_instance["ChassisName"], "Ambient", "Cel", metrics.dict["TemperatureSummaryCelsius"], chassis_instance["Readings"] )

                # Add information for each fan reported
                if "Fans" in thermal.dict:
                    fans = context.get( thermal.dict["Fans"]["@odata.id"] )
                    for fan_member in fans.dict["Members"]:
                        fan = context.get( fan_member["@odata.id"] )
                        fan_name = "Fan " + fan.dict["Id"]
                        get_discrete_status( fan_name + " State", fan.dict, chassis_instance["Readings"] )
                        get_excerpt_status( fan_name, "SpeedPercent", "%", fan.dict, chassis_instance["Readings"])

                # Add information for each of the redundancy groups reported
                if "FanRedundancy" in thermal.dict:
                    for i, redundancy in enumerate( thermal.dict["FanRedundancy"] ):
                        get_discrete_status( "Fan Redundancy " + str( i ), redundancy, chassis_instance["Readings"] )

            # Get all sensor readings if available
            if "Sensors" in chassis.dict:
                sensors = context.get( chassis.dict["Sensors"]["@odata.id"] )
                for sensor_member in sensors.dict["Members"]:
                    sensor = context.get( sensor_member["@odata.id"] )
                    get_sensor_status( sensor.dict, chassis_instance["Readings"] )

        # Older power/thermal models
        else:
            # Get readings from the Power resource if available
            if "Power" in chassis.dict:
                power = context.get( chassis.dict["Power"]["@odata.id"] )

                # Add information for each power supply reported
                if "PowerSupplies" in power.dict:
                    for power_supply in power.dict["PowerSupplies"]:
                        power_supply_name = "Power Supply " + power_supply["MemberId"]
                        if "Name" in power_supply:
                            power_supply_name = power_supply["Name"]
                        get_discrete_status( power_supply_name + " State", power_supply, chassis_instance["Readings"] )
                        get_analog_status_small( power_supply_name, "ReadingVolts", "V", power_supply, chassis_instance["Readings"] )
                        get_analog_status_small( power_supply_name, "LineInputVoltage", "V", power_supply, chassis_instance["Readings"] )
                        get_analog_status_small( power_supply_name, "PowerCapacityWatts", "W", power_supply, chassis_instance["Readings"] )
                        get_analog_status_small( power_supply_name, "LastPowerOutputWatts", "W", power_supply, chassis_instance["Readings"] )

                # Add information for each of the voltages reported
                if "Voltages" in power.dict:
                    for voltage in power.dict["Voltages"]:
                        voltage_name = "Voltage " + voltage["MemberId"]
                        if "Name" in voltage:
                            voltage_name = voltage["Name"]
                        get_analog_status_full( voltage_name, voltage, chassis_instance["Readings"] )

                # Add information for each of the redundancy groups reported
                if "Redundancy" in power.dict:
                    for redundancy in power.dict["Redundancy"]:
                        get_discrete_status( redundancy["Name"], redundancy, chassis_instance["Readings"] )

            # Get readings from the Thermal resource if available
            if "Thermal" in chassis.dict:
                thermal = context.get( chassis.dict["Thermal"]["@odata.id"] )

                # Add information for each of the temperatures reported
                if "Temperatures" in thermal.dict:
                    for temperature in thermal.dict["Temperatures"]:
                        temperature_name = "Temperature " + temperature["MemberId"]
                        if "Name" in temperature:
                            temperature_name = temperature["Name"]
                        get_analog_status_full( temperature_name, temperature, chassis_instance["Readings"] )

                # Add information for each of the fans reported
                if "Fans" in thermal.dict:
                    for fan in thermal.dict["Fans"]:
                        fan_name = "Fan " + fan["MemberId"]
                        if "Name" in fan:
                            fan_name = fan["Name"]
                        get_analog_status_full( fan_name, fan, chassis_instance["Readings"] )

                # Add information for each of the redundancy groups reported
                if "Redundancy" in thermal.dict:
                    for redundancy in thermal.dict["Redundancy"]:
                        get_discrete_status( redundancy["Name"], redundancy, chassis_instance["Readings"] )

    return sensor_list

def get_discrete_status( name, object, readings ):
    """
    Builds the status reading based on the Status property

    Args:
        name: The name to apply to the reading
        object: The object to parse
        readings: The list of readings to update
    """

    if "Status" not in object:
        # Do not produce a dummy reading if Status is not found
        return
    state, health = get_status( object )

    reading = {
        "Name": name,
        "Reading": state,
        "Units": None,
        "State": state,
        "Health": health,
        "LowerFatal": None,
        "LowerCritical": None,
        "LowerCaution": None,
        "UpperCaution": None,
        "UpperCritical": None,
        "UpperFatal": None,
    }

    readings.append( reading )

def get_analog_status_full( name, object, readings ):
    """
    Builds a full analog reading based on the the contents of an object

    Args:
        name: The name to apply to the reading
        object: The object to parse
        readings: The list of readings to update
    """

    state, health = get_status( object )
    units = None

    if "ReadingCelsius" in object:
        reading_val = object["ReadingCelsius"]
        units = "Cel"
    elif "ReadingVolts" in object:
        reading_val = object["ReadingVolts"]
        units = "V"
    elif "Reading" in object:
        reading_val = object["Reading"]
        if "ReadingUnits" in object:
            if object["ReadingUnits"] == "Percent":
                units = "%"
            else:
                units = object["ReadingUnits"]
    else:
        reading_val = None

    if reading_val is None:
        reading_val = state

    reading = {
        "Name": name,
        "Reading": reading_val,
        "Units": units,
        "State": state,
        "Health": health,
        "LowerFatal": object.get( "LowerThresholdFatal", None ),
        "LowerCritical": object.get( "LowerThresholdCritical", None ),
        "LowerCaution": object.get( "LowerThresholdNonCritical", None ),
        "UpperCaution": object.get( "UpperThresholdNonCritical", None ),
        "UpperCritical": object.get( "UpperThresholdCritical", None ),
        "UpperFatal": object.get( "UpperThresholdFatal", None ),
    }

    readings.append( reading )

def get_analog_status_small( name, field, units, object, readings ):
    """
    Builds an analog reading without thresholds based on a single field

    Args:
        name: The name to apply to the reading
        field: The field with the reading
        units: The units of measure for the reading
        object: The object to parse
        readings: The list of readings to update
    """

    if field not in object:
        return

    reading = {
        "Name": "{} {}".format( name, field ),
        "Reading": object[field],
        "Units": units,
        "State": None,
        "Health": None,
        "LowerFatal": None,
        "LowerCritical": None,
        "LowerCaution": None,
        "UpperCaution": None,
        "UpperCritical": None,
        "UpperFatal": None,
    }

    readings.append( reading )

def get_excerpt_status( name, field, units, object, readings ):
    """
    Builds an analog reading based on an excerpt

    Args:
        name: The name to apply to the reading
        field: The field with the reading
        units: The units of measure for the reading
        object: The object to parse
        readings: The list of readings to update
    """

    if field not in object:
        return

    if isinstance( object[field], list ):
        for i, item in enumerate( object[field] ):
            if "DataSourceUri" not in item:
                reading = {
                    "Name": "{} {} {}".format( name, field, i ),
                    "Reading": item["Reading"],
                    "Units": units,
                    "State": None,
                    "Health": None,
                    "LowerFatal": None,
                    "LowerCritical": None,
                    "LowerCaution": None,
                    "UpperCaution": None,
                    "UpperCritical": None,
                    "UpperFatal": None,
                }
                readings.append( reading )
    else:
        if "DataSourceUri" not in object[field]:
            reading = {
                "Name": "{} {}".format( name, field ),
                "Reading": object[field]["Reading"],
                "Units": units,
                "State": None,
                "Health": None,
                "LowerFatal": None,
                "LowerCritical": None,
                "LowerCaution": None,
                "UpperCaution": None,
                "UpperCritical": None,
                "UpperFatal": None,
            }
            readings.append( reading )

def get_sensor_status( sensor, readings ):
    """
    Builds an analog reading from a sensor

    Args:
        sensor: The sensor
        readings: The list of readings to update
    """

    state, health = get_status( sensor )
    units = sensor.get( "ReadingUnits", None )
    reading_val = sensor.get( "Reading", None )
    if reading_val is None:
        reading_val = state

    reading = {
        "Name": sensor["Name"],
        "Reading": reading_val,
        "Units": units,
        "State": state,
        "Health": health,
        "LowerFatal": sensor.get( "Thresholds", {} ).get( "LowerFatal", {} ).get( "Reading", None ),
        "LowerCritical": sensor.get( "Thresholds", {} ).get( "LowerCritical", {} ).get( "Reading", None ),
        "LowerCaution": sensor.get( "Thresholds", {} ).get( "LowerCaution", {} ).get( "Reading", None ),
        "UpperCaution": sensor.get( "Thresholds", {} ).get( "UpperCaution", {} ).get( "Reading", None ),
        "UpperCritical": sensor.get( "Thresholds", {} ).get( "UpperCritical", {} ).get( "Reading", None ),
        "UpperFatal": sensor.get( "Thresholds", {} ).get( "UpperFatal", {} ).get( "Reading", None ),
    }

    readings.append( reading )

def get_status( object ):
    """
    Gets the lower status information

    Args:
        object: The object containing the Status property

    Returns:
        The state within Status (None if not found)
        The health within Status (None if not found)
    """

    state = None
    health = None

    if "Status" in object:
        state = object["Status"].get( "State", None )
        health = object["Status"].get( "Health", None )

    return state, health

def print_sensors( sensor_list ):
    """
    Prints the sensor list into a table

    Args:
        sensor_list: The sensor list to print
    """

    sensor_line_format = "  {:25s} | {:10s} | {:8s} | {:8s} | {:8s} | {:8s} | {:8s} | {:8s} | {:8s}"

    # Go through each chassis object in the list
    for chassis in sensor_list:

        print( "Chassis '" + chassis["ChassisName"] + "' Status" )
        print( sensor_line_format.format( "Sensor", "Reading", "Health", "LF", "LC", "LNC", "UNC", "UC", "UF" ) )

        # Go through each reading in the chassis
        for reading in chassis["Readings"]:

            # Sanitize the data for printing; use a new object to not modify the original reading data
            reading_pr = {}
            for item in reading:
                if item == "Name":
                    reading_pr["Name"] = reading["Name"][:25]
                elif item == "Reading":
                    if reading["Reading"] is None:
                        reading_pr["Reading"] = "Unknown"
                    else:
                        reading_pr["Reading"] = str( reading["Reading"] )
                        if reading["Units"] is not None:
                            reading_pr["Reading"] = reading_pr["Reading"] + reading["Units"]
                    reading_pr["Reading"] = reading_pr["Reading"][:10]
                elif item == "Units":
                    if reading["Units"] is None:
                        reading_pr["Units"] = ""
                else:
                    if reading[item] is None:
                        reading_pr[item] = "N/A"
                    else:
                        reading_pr[item] = str( reading[item] )

            # Print it
            print( sensor_line_format.format( reading_pr["Name"], reading_pr["Reading"], reading_pr["Health"],
                reading_pr["LowerFatal"], reading_pr["LowerCritical"], reading_pr["LowerCaution"],
                reading_pr["UpperCaution"], reading_pr["UpperCritical"], reading_pr["UpperFatal"] ) )
        print( "" )



#! /usr/bin/python
# Copyright Notice:
# Copyright 2021 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/master/LICENSE.md

"""
Redfish Virtual Media

File : rf_virtual_media.py

Brief : This script uses the redfish_utilities module to manage virtual media
"""

import argparse
import redfish
import redfish_utilities

# Get the input arguments
argget = argparse.ArgumentParser( description = "A tool to manage virtual media of a system" )
argget.add_argument( "--user", "-u", type = str, required = True, help = "The user name for authentication" )
argget.add_argument( "--password", "-p",  type = str, required = True, help = "The password for authentication" )
argget.add_argument( "--rhost", "-r", type = str, required = True, help = "The address of the Redfish service (with scheme)" )
argget.add_argument( "--system", "-s", type = str, help = "The ID of the system containing the virtual media" )
subparsers = argget.add_subparsers( dest = "command" )
info_argget = subparsers.add_parser( "info", help = "Displays information about the virtual media for a system" )
insert_argget = subparsers.add_parser( "insert", help = "Inserts virtual media for a system" )
insert_argget.add_argument( "--image", "-image", type = str, required = True, help = "The URI of the image to insert" )
insert_argget.add_argument( "--id", "-i", type = str, help = "The identifier of the virtual media instance to insert" )
insert_argget.add_argument( "--notinserted", "-notinserted", action = "store_false", help = "Indicates if the media is to be marked as not inserted for the system", default = None )
insert_argget.add_argument( "--writable", "-writable", action = "store_false", help = "Indicates if the media is to be marked as writable for the system", default = None )
insert_argget.add_argument( "--mediatypes", "-mt", type = str, nargs = '+', help = "A list of acceptable media types for the virtual media" )
eject_argget = subparsers.add_parser( "eject", help = "Ejects virtual media from a system" )
eject_argget.add_argument( "--id", "-i", type = str, required = True, help = "The identifier of the virtual media instance to eject" )
args = argget.parse_args()

# Set up the Redfish object
redfish_obj = redfish.redfish_client( base_url = args.rhost, username = args.user, password = args.password )
redfish_obj.login( auth = "session" )

try:
    if args.command == "insert":
        print( "Inserting '{}'".format( args.image ) )
        redfish_utilities.insert_virtual_media( redfish_obj, args.image, args.system, args.id, args.mediatypes, args.notinserted, args.writable )
    elif args.command == "eject":
        print( "Ejecting '{}'".format( args.id ) )
        redfish_utilities.eject_virtual_media (redfish_obj, args.id, args.system )
    else:
        virtual_media = redfish_utilities.get_virtual_media( redfish_obj, args.system )
        redfish_utilities.print_virtual_media( virtual_media )
finally:
    # Log out
    redfish_obj.logout()



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2020 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/master/LICENSE.md

"""
Redfish Event Service

File : rf_event_service.py

Brief : This script uses the redfish_utilities module to manage the event service and event subscriptions
"""

import argparse
import redfish
import redfish_utilities

# Get the input arguments
argget = argparse.ArgumentParser( description = "A tool to manage the event service on a Redfish service" )
argget.add_argument( "--user", "-u", type = str, required = True, help = "The user name for authentication" )
argget.add_argument( "--password", "-p",  type = str, required = True, help = "The password for authentication" )
argget.add_argument( "--rhost", "-r", type = str, required = True, help = "The address of the Redfish service (with scheme)" )
subparsers = argget.add_subparsers( dest = "command" )
info_argget = subparsers.add_parser( "info", help = "Displays information about the event service and subscriptions" )
sub_argget = subparsers.add_parser( "subscribe", help = "Creates an event subscription to a specified URL" )
sub_argget.add_argument( "--destination", "-dest", type = str, required = True, help = "The URL where events are sent for the subscription" )
sub_argget.add_argument( "--context", "-c", type = str, help = "The context string for the subscription that is supplied back in the event payload" )
sub_argget.add_argument( "--expand", "-e", action = "store_true", help = "Indicates if the origin of condition in the event is to be expanded", default = None )
sub_argget.add_argument( "--format", "-f", type = str, help = "The format of the event payloads" )
sub_argget.add_argument( "--resourcetypes", "-rt", type = str, nargs = '+', help = "A list of resource types for the subscription" )
sub_argget.add_argument( "--registries", "-reg", type = str, nargs = '+', help = "A list of registries for the subscription" )
sub_argget.add_argument( "--eventtypes", "-et", type = str, nargs = '+', help = "A list of event types for the subscription; this option has been deprecated in favor of other methods such as 'resource types' and 'registries'" )
unsub_argget = subparsers.add_parser( "unsubscribe", help = "Deletes an event subscription" )
unsub_argget.add_argument( "--id", "-i", type = str, required = True, help = "The identifier of the event subscription to be deleted" )
args = argget.parse_args()

# Set up the Redfish object
redfish_obj = redfish.redfish_client( base_url = args.rhost, username = args.user, password = args.password )
redfish_obj.login( auth = "session" )

try:
    if args.command == "subscribe":
        response = redfish_utilities.create_event_subscription( redfish_obj, args.destination, format = args.format, client_context = args.context,
            expand = args.expand, resource_types = args.resourcetypes, registries = args.registries )
        print( "Created subscription '{}'".format( response.getheader( "Location" ) ) )
    elif args.command == "unsubscribe":
        print( "Deleting subscription '{}'".format( args.id ) )
        redfish_utilities.delete_event_subscription( redfish_obj, args.id )
    else:
        event_service = redfish_utilities.get_event_service( redfish_obj )
        redfish_utilities.print_event_service( event_service )
        print( "" )
        event_subscriptions = redfish_utilities.get_event_subscriptions( redfish_obj )
        redfish_utilities.print_event_subscriptions( event_subscriptions )
finally:
    # Log out
    redfish_obj.logout()



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2020 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/master/LICENSE.md

"""
Redfish Power Reset

File : rf_power_reset.py

Brief : This script uses the redfish_utilities module to perform a reset of the system
"""

import argparse
import redfish
import redfish_utilities

# Get the input arguments
argget = argparse.ArgumentParser( description = "A tool to perform a power/reset operation of a system" )
argget.add_argument( "--user", "-u", type = str, required = True, help = "The user name for authentication" )
argget.add_argument( "--password", "-p",  type = str, required = True, help = "The password for authentication" )
argget.add_argument( "--rhost", "-r", type = str, required = True, help = "The address of the Redfish service (with scheme)" )
argget.add_argument( "--system", "-s", type = str, help = "The ID of the system to reset" )
argget.add_argument( "--type", "-t", type = str, help = "The type of power/reset operation to perform", choices = redfish_utilities.reset_types )
argget.add_argument( "--info", "-info", action = "store_true", help = "Indicates if reset and power information should be reported" )
args = argget.parse_args()

# Set up the Redfish object
redfish_obj = redfish.redfish_client( base_url = args.rhost, username = args.user, password = args.password )
redfish_obj.login( auth = "session" )

try:
    if args.info:
        system_info = redfish_utilities.get_system( redfish_obj, args.system )
        reset_uri, reset_parameters = redfish_utilities.get_system_reset_info( redfish_obj, args.system, system_info )
        printed_reset_types = False
        for param in reset_parameters:
            if param["Name"] == "ResetType" and "AllowableValues" in param:
                print( "Supported reset types: {}".format( ", ".join( param["AllowableValues"] ) ) )
                printed_reset_types = True
        if not printed_reset_types:
            print( "No reset information found" )
        if "PowerState" in system_info.dict:
            print( "Current power state: {}".format( system_info.dict["PowerState"] ) )
        else:
            print( "No power state information found")
    else:
        print( "Resetting the system..." )
        response = redfish_utilities.system_reset( redfish_obj, args.system, args.type )
        response = redfish_utilities.poll_task_monitor( redfish_obj, response )
        redfish_utilities.verify_response( response )
finally:
    # Log out
    redfish_obj.logout()



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2021 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/master/LICENSE.md

"""
Managers Module

File : managers.py

Brief : This file contains the definitions and functionalities for interacting
        with the managers collection for a given Redfish service
"""

from .messages import verify_response
from .resets import reset_types

class RedfishManagerNotFoundError( Exception ):
    """
    Raised when a matching manager cannot be found
    """
    pass

class RedfishManagerEthIntNotFoundError( Exception ):
    """
    Raised when a matching Ethernet interface cannot be found
    """
    pass

class RedfishManagerResetNotFoundError( Exception ):
    """
    Raised when the Reset action cannot be found
    """
    pass

def get_manager_ids( context ):
    """
    Finds the manager collection and returns all of the member's identifiers

    Args:
        context: The Redfish client object with an open session

    Returns:
        A list of identifiers of the members of the manager collection
    """

    # Get the service root to find the manager collection
    service_root = context.get( "/redfish/v1/" )
    if "Managers" not in service_root.dict:
        # No manager collection
        raise RedfishManagerNotFoundError( "Service does not contain a manager collection" )

    # Get the manager collection and iterate through its collection
    avail_managers = []
    manager_col = context.get( service_root.dict["Managers"]["@odata.id"] )
    while True:
        for manager_member in manager_col.dict["Members"]:
            avail_managers.append( manager_member["@odata.id"].strip( "/" ).split( "/" )[-1] )
        if "Members@odata.nextLink" not in manager_col.dict:
            break
        manager_col = context.get( manager_col.dict["Members@odata.nextLink"] )
    return avail_managers

def get_manager( context, manager_id = None ):
    """
    Finds a manager matching the given identifier and returns its resource

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager

    Returns:
        The manager resource
    """

    manager_uri_pattern = "/redfish/v1/Managers/{}"
    avail_managers = None

    # If given an identifier, get the manager directly
    if manager_id is not None:
        manager = context.get( manager_uri_pattern.format( manager_id ) )
    # No identifier given; see if there's exactly one member
    else:
        avail_managers = get_manager_ids( context )
        if len( avail_managers ) == 1:
            manager = context.get( manager_uri_pattern.format( avail_managers[0] ) )
        else:
            raise RedfishManagerNotFoundError( "Service does not contain exactly one manager; a target manager needs to be specified: {}".format( ", ".join( avail_managers ) ) )

    # Check the response and return the manager if the response is good
    try:
        verify_response( manager )
    except:
        if avail_managers is None:
            avail_managers = get_manager_ids( context )
        raise RedfishManagerNotFoundError( "Service does not contain a manager called {}; valid managers: {}".format( manager_id, ", ".join( avail_managers ) ) ) from None
    return manager

def print_manager( manager ):
    """
    Prints the manager info

    Args:
        manager: The manager info to print
    """

    manager_line_format = "  {}: {}"
    manager_properties = [ "Status", "ManagerType", "PowerState", "FirmwareVersion", "DateTime", "DateTimeLocalOffset", "LastResetTime",
        "UUID", "ServiceEntryPointUUID", "Manufacturer", "Model", "PartNumber", "SparePartNumber", "SerialNumber" ]
    print( "Manager {} Info".format( manager.dict["Id"] ) )
    for property in manager_properties:
        if property in manager.dict:
            prop_val = manager.dict[property]
            if isinstance( prop_val, list ):
                prop_val = ", ".join( prop_val )
            elif property == "Status":
                prop_val = "State: {}, Health: {}".format( prop_val.get( "State", "N/A" ), prop_val.get( "Health", "N/A" ) )
            print( manager_line_format.format( property, prop_val ) )
    print( "" )

def get_manager_reset_info( context, manager_id = None, manager = None ):
    """
    Finds a manager matching the given ID and returns its reset info

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        manager: Existing manager resource to inspect for reset info

    Returns:
        The URI of the Reset action
        A list of parameter requirements from the Action Info
    """

    if manager is None:
        manager = get_manager( context, manager_id )

    # Check that there is a Reset action
    if "Actions" not in manager.dict:
        raise RedfishManagerResetNotFoundError( "Manager does not support Reset" )
    if "#Manager.Reset" not in manager.dict["Actions"]:
        raise RedfishManagerResetNotFoundError( "Manager does not support Reset" )

    # Extract the info about the SimpleUpdate action
    reset_action = manager.dict["Actions"]["#Manager.Reset"]
    reset_uri = reset_action["target"]

    if "@Redfish.ActionInfo" not in reset_action:
        # No Action Info; need to build this manually based on other annotations

        # Default parameter requirements
        reset_parameters = [
            {
                "Name": "ResetType",
                "Required": False,
                "DataType": "String",
                "AllowableValues": reset_types
            }
        ]

        # Get the AllowableValues from annotations
        for param in reset_parameters:
            if param["Name"] + "@Redfish.AllowableValues" in reset_action:
                param["AllowableValues"] = reset_action[param["Name"] + "@Redfish.AllowableValues"]
    else:
        # Get the Action Info and its parameter listing
        action_info = context.get( reset_action["@Redfish.ActionInfo"] )
        reset_parameters = action_info.dict["Parameters"]

    return reset_uri, reset_parameters

def manager_reset( context, manager_id = None, reset_type = None ):
    """
    Finds a manager matching the given ID and performs a reset

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        reset_type: The type of reset to perform; if None, perform one of the common resets

    Returns:
        The response of the action
    """

    # Check that the values themselves are supported by the schema
    reset_type_values = reset_types
    if reset_type is not None:
        if reset_type not in reset_type_values:
            raise ValueError( "{} is not an allowable reset type ({})".format( reset_type, ", ".join( reset_type_values ) ) )

    # Locate the reset action
    reset_uri, reset_parameters = get_manager_reset_info( context, manager_id )

    # Build the payload
    if reset_type is None:
        for param in reset_parameters:
            if param["Name"] == "ResetType":
                if "GracefulRestart" in param["AllowableValues"]:
                    reset_type = "GracefulRestart"
                elif "ForceRestart" in param["AllowableValues"]:
                    reset_type = "ForceRestart"
                elif "PowerCycle" in param["AllowableValues"]:
                    reset_type = "PowerCycle"

    payload = {}
    if reset_type is not None:
        payload["ResetType"] = reset_type

    # Reset the manager
    response = context.post( reset_uri, body = payload )
    verify_response( response )
    return response

def get_manager_ethernet_interface_ids( context, manager_id = None ):
    """
    Finds the Ethernet interface collection for a manager and returns all of the member's identifiers

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager

    Returns:
        A list of identifiers of the members of the Ethernet interface collection
    """

    # Get the manager to find its Ethernet interface collection
    manager = get_manager( context, manager_id )
    if "EthernetInterfaces" not in manager.dict:
        # No Ethernet interface collection
        raise RedfishManagerEthIntNotFoundError( "Manager {} does not contain an Ethernet interface collection".format( manager.dict["Id"] ) )

    # Get the Ethernet interface collection and iterate through its collection
    avail_interfaces = []
    interface_col = context.get( manager.dict["EthernetInterfaces"]["@odata.id"] )
    while True:
        for interface_member in interface_col.dict["Members"]:
            avail_interfaces.append( interface_member["@odata.id"].strip( "/" ).split( "/" )[-1] )
        if "Members@odata.nextLink" not in interface_col.dict:
            break
        interface_col = context.get( interface_col.dict["Members@odata.nextLink"] )
    return avail_interfaces

def get_manager_ethernet_interface( context, manager_id = None, interface_id = None ):
    """
    Finds an Ethernet interface for a manager matching the given identifiers and returns its resource

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        interface_id: The Ethernet interface to locate; if None, perform on the only Ethernet interface

    Returns:
        The manager resource
    """

    interface_uri_pattern = "/redfish/v1/Managers/{}/EthernetInterfaces/{}"
    avail_interfaces = None

    # Get the manager identifier in order to build the full URI later
    if manager_id is None:
        manager = get_manager( context, None )
        manager_id = manager.dict["Id"]

    # If given an identifier, get the Ethernet interface directly
    if interface_id is not None:
        interface = context.get( interface_uri_pattern.format( manager_id, interface_id ) )
    # No identifier given; see if there's exactly one member
    else:
        avail_interfaces = get_manager_ethernet_interface_ids( context, manager_id )
        if len( avail_interfaces ) == 1:
            interface = context.get( interface_uri_pattern.format( manager_id, avail_interfaces[0] ) )
        else:
            raise RedfishManagerEthIntNotFoundError( "Manager {} does not contain exactly one Ethernet interface; a target Ethernet interface needs to be specified: {}".format( manager_id, ", ".join( avail_interfaces ) ) )

    # Check the response and return the Ethernet interface if the response is good
    try:
        verify_response( interface )
    except:
        if avail_interfaces is None:
            avail_interfaces = get_manager_ethernet_interface_ids( context, manager_id )
        raise RedfishManagerEthIntNotFoundError( "Manager {} does not contain an Ethernet interface called {}; valid Ethernet interfaces: {}".format( manager_id, interface_id, ", ".join( avail_interfaces ) ) ) from None
    return interface

def set_manager_ethernet_interface( context, manager_id = None, interface_id = None, vlan = None, ipv4_addresses = None, dhcpv4 = None, ipv6_addresses = None, ipv6_gateways = None, dhcpv6 = None ):
    """
    Finds an Ethernet interface matching the given ID and updates specified properties

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        interface_id: The Ethernet interface to locate; if None, perform on the only interface
        vlan: An object containing new VLAN settings for the interface
        ipv4_addresses: An array of objects containing new IPv4 static addresses for the interface
        dhcpv4: An object containing new DHCPv4 settings for the interface
        ipv6_addresses: An array of objects containing new IPv6 static addresses for the interface
        ipv6_gateways: New IPv6 static gateways for the interface
        dhcpv6: An object containing new DHCPv6 settings for the interface

    Returns:
        The response of the PATCH
    """

    # Locate the interface
    interface = get_manager_ethernet_interface( context, manager_id, interface_id )

    # Update the interface based on the request parameters
    payload = {}
    if vlan is not None:
        payload["VLAN"] = vlan
    if ipv4_addresses is not None:
        if "IPv4StaticAddresses" in interface.dict:
            payload["IPv4StaticAddresses"] = ipv4_addresses
        else:
            # Older services manage static addresses through the "assigned addresses" property
            payload["IPv4Addresses"] = ipv4_addresses
    if dhcpv4 is not None:
        payload["DHCPv4"] = dhcpv4
    if ipv6_addresses is not None:
        payload["IPv6StaticAddresses"] = ipv6_addresses
    if ipv6_gateways is not None:
        payload["IPv6StaticDefaultGateways"] = ipv6_gateways
    if dhcpv6 is not None:
        payload["DHCPv6"] = dhcpv6
    headers = None
    etag = interface.getheader( "ETag" )
    if etag is not None:
        headers = { "If-Match": etag }
    response = context.patch( interface.dict["@odata.id"], body = payload, headers = headers )
    verify_response( response )
    return response

def print_manager_ethernet_interface( interface ):
    """
    Prints the Ethernet interface info

    Args:
        interface: The Ethernet interface info to print
    """

    interface_line_format = "  {}: {}"
    interface_properties = [ "Status", "InterfaceEnabled", "LinkStatus", "MACAddress", "PermanentMACAddress", "SpeedMbps", "AutoNeg", "FullDuplex",
        "MTUSize", "HostName", "FQDN", "NameServers", "StaticNameServers" ]
    print( "Ethernet Interface {} Info".format( interface.dict["Id"] ) )
    for property in interface_properties:
        if property in interface.dict:
            prop_val = interface.dict[property]
            if isinstance( prop_val, list ):
                prop_val = ", ".join( prop_val )
            elif property == "Status":
                prop_val = "State: {}, Health: {}".format( prop_val.get( "State", "N/A" ), prop_val.get( "Health", "N/A" ) )
            print( interface_line_format.format( property, prop_val ) )
    print( "" )

    if "VLAN" in interface.dict:
        print( "  VLAN Info" )
        print( "    Enabled: {}".format( interface.dict["VLAN"].get( "VLANEnable", False ) ) )
        print( "    ID: {}".format( interface.dict["VLAN"].get( "VLANId", "N/A" ) ) )
        print( "    Priority: {}".format( interface.dict["VLAN"].get( "VLANPriority", "N/A" ) ) )
        print( "" )

    print( "  IPv4 Info" )
    if "DHCPv4" in interface.dict:
        print( "    DHCP Enabled: {}".format( interface.dict["DHCPv4"]["DHCPEnabled"] ) )
    if "IPv4Addresses" in interface.dict:
        print( "    Assigned Addresses" )
        for i, address in enumerate( interface.dict["IPv4Addresses"] ):
            if address is None:
                print( "      Empty" )
            elif i == 0:
                print( "      {}: {}, {}, {}".format( address.get( "Address", "N/A" ), address.get( "SubnetMask", "N/A" ), address.get( "Gateway", "N/A" ), address.get( "AddressOrigin", "N/A" ) ) )
            else:
                print( "      {}: {}, {}".format( address.get( "Address", "N/A" ), address.get( "SubnetMask", "N/A" ), address.get( "AddressOrigin", "N/A" ) ) )
    if "IPv4StaticAddresses" in interface.dict:
        print( "    Static Addresses" )
        for i, address in enumerate( interface.dict["IPv4StaticAddresses"] ):
            if address is None:
                print( "       Empty" )
            elif i == 0:
                print( "      {}: {}, {}".format( address.get( "Address", "N/A" ), address.get( "SubnetMask", "N/A" ), address.get( "Gateway", "N/A" ) ) )
            else:
                print( "      {}: {}".format( address["Address"], address["SubnetMask"] ) )
    print( "" )

    print( "  IPv6 Info" )
    if "DHCPv6" in interface.dict:
        print( "    DHCP Mode: {}".format( interface.dict["DHCPv6"]["OperatingMode"] ) )
    if "IPv6Addresses" in interface.dict:
        print( "    Assigned Addresses" )
        for address in interface.dict["IPv6Addresses"]:
            print( "      {}/{}: {}, {}".format( address.get( "Address", "N/A" ), address.get( "PrefixLength", "N/A" ), address.get( "AddressOrigin", "N/A" ), address.get( "AddressState", "N/A" ) ) )
    if "IPv6StaticAddresses" in interface.dict:
        print( "    Static Addresses" )
        for address in interface.dict["IPv6StaticAddresses"]:
            if address is None:
                print( "      Empty" )
            else:
                print( "      {}/{}".format( address.get( "Address", "N/A" ), address.get( "PrefixLength", "N/A" ) ) )
    if "IPv6StaticDefaultGateways" in interface.dict:
        print( "    Static Default Gateways" )
        for address in interface.dict["IPv6StaticDefaultGateways"]:
            if address is None:
                print( "      Empty" )
            else:
                print( "      {}/{}".format( address.get( "Address", "N/A" ), address.get( "PrefixLength", "N/A" ) ) )
    if "IPv6DefaultGateway" in interface.dict:
        print( "    Default Gateway: {}".format( interface.dict["IPv6DefaultGateway"] ) )
    if "IPv6AddressPolicyTable" in interface.dict:
        print( "    Address Policy Table" )
        for policy in interface.dict["IPv6AddressPolicyTable"]:
            if policy is None:
                print( "      Empty" )
            else:
                print( "      Prefix: {}, Prec: {}, Label: {}".format( policy.get( "Prefix", "N/A" ), policy.get( "Precedence", "N/A" ), policy.get( "Label", "N/A" ) ) )
    print( "" )


#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2022 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/master/LICENSE.md

from .accounts import get_users
from .accounts import print_users
from .accounts import add_user
from .accounts import delete_user
from .accounts import modify_user
from .event_service import get_event_service
from .event_service import print_event_service
from .event_service import get_event_subscriptions
from .event_service import print_event_subscriptions
from .event_service import create_event_subscription
from .event_service import delete_event_subscription
from .inventory import get_system_inventory
from .inventory import print_system_inventory
from .inventory import write_system_inventory
from .logs import log_container
from .logs import diagnostic_data_types
from .logs import get_log_entries
from .logs import print_log_entries
from .logs import clear_log_entries
from .logs import collect_diagnostic_data
from .logs import download_diagnostic_data
from .managers import get_manager_ids
from .managers import get_manager
from .managers import print_manager
from .managers import get_manager_reset_info
from .managers import manager_reset
from .managers import get_manager_ethernet_interface_ids
from .managers import get_manager_ethernet_interface
from .managers import set_manager_ethernet_interface
from .managers import print_manager_ethernet_interface
from .messages import print_error_payload
from .messages import verify_response
from .resets import reset_types
from .sensors import get_sensors
from .sensors import print_sensors
from .systems import get_system_ids
from .systems import get_system
from .systems import get_system_boot
from .systems import set_system_boot
from .systems import print_system_boot
from .systems import get_system_reset_info
from .systems import system_reset
from .systems import get_virtual_media
from .systems import print_virtual_media
from .systems import insert_virtual_media
from .systems import eject_virtual_media
from .systems import get_system_bios
from .systems import set_system_bios
from .systems import print_system_bios
from .tasks import poll_task_monitor
from .update import get_simple_update_info
from .update import simple_update



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2020 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/master/LICENSE.md

"""
Update Module

File : update.py

Brief : This file contains the definitions and functionalities for interacting
        with the UpdateService for a given Redfish service
"""

from .messages import verify_response

class RedfishUpdateServiceNotFoundError( Exception ):
    """
    Raised when the Update Service or an update action cannot be found
    """
    pass

def get_simple_update_info( context ):
    """
    Locates the SimpleUpdate action and collects its information

    Args:
        context: The Redfish client object with an open session

    Returns:
        The URI of the Simple Update action
        A list of parameter requirements from the Action Info
    """

    # Get the Update Service
    update_service = get_update_service( context )

    # Check that there is a SimpleUpdate action
    if "Actions" not in update_service.dict:
        raise RedfishUpdateServiceNotFoundError( "Service does not support SimpleUpdate" )
    if "#UpdateService.SimpleUpdate" not in update_service.dict["Actions"]:
        raise RedfishUpdateServiceNotFoundError( "Service does not support SimpleUpdate" )

    # Extract the info about the SimpleUpdate action
    simple_update_action = update_service.dict["Actions"]["#UpdateService.SimpleUpdate"]
    simple_update_uri = simple_update_action["target"]

    if "@Redfish.ActionInfo" not in simple_update_action:
        # No Action Info; need to build this manually based on other annotations

        # Default parameter requirements
        simple_update_parameters = [
            {
                "Name": "ImageURI",
                "Required": True,
                "DataType": "String"
            },
            {
                "Name": "TransferProtocol",
                "Required": False,
                "DataType": "String",
                "AllowableValues": [ "CIFS", "FTP", "SFTP", "HTTP", "HTTPS", "NSF", "SCP", "TFTP", "OEM", "NFS" ]
            },
            {
                "Name": "Targets",
                "Required": False,
                "DateType": "StringArray"
            },
            {
                "Name": "Username",
                "Required": False,
                "DataType": "String"
            },
            {
                "Name": "Password",
                "Required": False,
                "DataType": "String"
            }
        ]

        # Get the AllowableValues from annotations
        for param in simple_update_parameters:
            if param["Name"] + "@Redfish.AllowableValues" in simple_update_action:
                param["AllowableValues"] = simple_update_action[param["Name"] + "@Redfish.AllowableValues"]
    else:
        # Get the Action Info and its parameter listing
        action_info = context.get( simple_update_action["@Redfish.ActionInfo"] )
        simple_update_parameters = action_info.dict["Parameters"]

    return simple_update_uri, simple_update_parameters

def simple_update( context, image_uri, protocol = None, targets = None, username = None, password = None ):
    """
    Performs a Simple Update request

    Args:
        context: The Redfish client object with an open session
        image_uri: The image URI for the update
        protocol: The transfer protocol for the update
        targets: The targets receiving the update
        username: The username for retrieving the update for the given URI
        password: The password for retrieving the update for the given URI

    Returns:
        The response from the request
    """

    # Get the Simple Update info
    uri, params = get_simple_update_info( context )

    # Build the request body
    body = {
        "ImageURI": image_uri
    }
    if protocol is not None:
        body["TransferProtocol"] = protocol
    if targets is not None:
        body["Targets"] = targets
    if username is not None:
        body["Username"] = username
    if password is not None:
        body["Password"] = password

    response = context.post( uri, body = body )
    verify_response( response )
    return response

def get_update_service( context ):
    """
    Locates and gets the UpdateService resource

    Args:
        context: The Redfish client object with an open session

    Returns:
        The UpdateService resource
    """

    # Get the Service Root to find the Update Service
    service_root = context.get( "/redfish/v1/" )
    if "UpdateService" not in service_root.dict:
        # No Update Service
        raise RedfishUpdateServiceNotFoundError( "Service does not have an UpdateService" )

    return context.get( service_root.dict["UpdateService"]["@odata.id"] )



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2020 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/master/LICENSE.md

"""
Event Service Module

File : event_service.py

Brief : This file contains the definitions and functionalities for managing
        accounts on a Redfish Service
"""

from .messages import verify_response

class RedfishEventServiceNotFoundError( Exception ):
    """
    Raised when the event service cannot be found
    """
    pass

class RedfishEventSubscriptionNotFoundError( Exception ):
    """
    Raised when the specified event subscription cannot be found
    """
    pass

def get_event_service( context ):
    """
    Collects the event service information from a Redfish service

    Args:
        context: The Redfish client object with an open session

    Returns:
        An object containing information about the event service
    """

    # Get the service root to find the event service
    service_root = context.get( "/redfish/v1/" )
    if "EventService" not in service_root.dict:
        # No event service
        raise RedfishEventServiceNotFoundError( "Service does not contain an event service" )

    # Get the event service
    event_service = context.get( service_root.dict["EventService"]["@odata.id"] )
    return event_service.dict

def print_event_service( service ):
    """
    Prints the event service information from a Redfish service

    Args:
        service: The event service object
    """

    print( "Service Info" )

    # General status
    status = "Enabled"
    if "ServiceEnabled" in service:
        if not service["ServiceEnabled"]:
            status = "Disabled"
    if "Status" in service:
        if "State" in service["Status"]:
            status = service["Status"]["State"]
    print( "  Status: {}".format( status ) )

    # Delivery retry policy
    retry_policy = ""
    if "DeliveryRetryAttempts" in service:
        retry_policy += "{} attempts, ".format( service["DeliveryRetryAttempts"] )
    if "DeliveryRetryIntervalSeconds" in service:
        retry_policy += "{} second intervals, ".format( service["DeliveryRetryIntervalSeconds"] )
    if len( retry_policy ) > 2:
        retry_policy = retry_policy[:-2]
    else:
        retry_policy = "Unknown/Unspecified"
    print( "  Delivery Retry Policy: {}".format( retry_policy ) )

    # Subscription methods
    print( "  Event Types: {}".format( ", ".join( service.get( "EventTypesForSubscription", [ "N/A" ] ) ) ) )
    print( "  Event Formats: {}".format( ", ".join( service.get( "EventFormatTypes", [ "N/A" ] ) ) ) )
    print( "  Registries: {}".format( ", ".join( service.get( "RegistryPrefixes", [ "N/A" ] ) ) ) )
    print( "  Resource Types: {}".format( ", ".join( service.get( "ResourceTypes", [ "N/A" ] ) ) ) )
    print( "  Include Origin of Condition Supported: {}".format( service.get( "IncludeOriginOfConditionSupported", False ) ) )

    # SSE info
    print( "  SSE URI: {}".format( service.get( "ServerSentEventUri", "Not supported" ) ) )
    sse_filters = ""
    for filter in service.get( "SSEFilterPropertiesSupported", {} ):
        if filter == "EventType":
            # This style is deprecated
            continue
        if service["SSEFilterPropertiesSupported"][filter]:
            sse_filters += "{}, ".format( filter )
    if len( sse_filters ) > 2:
        sse_filters = sse_filters[:-2]
    else:
        sse_filters = "Unknown/Unspecified"
    print( "  SSE Filter Parameters: {}".format( sse_filters ) )

def get_event_subscriptions( context ):
    """
    Collects the event subscription information from a Redfish service

    Args:
        context: The Redfish client object with an open session

    Returns:
        A list containing all event subscriptions
    """

    # Get the event service
    event_service = get_event_service( context )
    if "Subscriptions" not in event_service:
        # No subscriptions
        raise RedfishEventServiceNotFoundError( "Service does not contain a subscription collection" )

    # Get each of the event subscriptions
    subscriptions = []
    subscription_col = context.get( event_service["Subscriptions"]["@odata.id"] )
    for subscription_member in subscription_col.dict["Members"]:
        subscription = context.get( subscription_member["@odata.id"] )
        subscriptions.append( subscription.dict )
    return subscriptions

def print_event_subscriptions( subscriptions ):
    """
    Prints the event subscription information from a Redfish service

    Args:
        subscriptions: An array of event subscription objects
    """

    subscription_line_format = "  {:36s} | {}: {}"

    print( "Subscription Info" )

    if len( subscriptions ) == 0:
        print( "  No subscriptions" )
        return

    for subscription in subscriptions:
        print( subscription_line_format.format( subscription["Id"], "Destination", subscription.get( "Destination", "N/A" ) ) )
        print( subscription_line_format.format( "", "State", subscription.get( "Status", {} ).get( "State", "Enabled" ) ) )
        if "Context" in subscription:
            print( subscription_line_format.format( "", "Context", subscription["Context"] ) )
        print( subscription_line_format.format( "", "Event Format", subscription.get( "EventFormatType", "Event" ) ) )
        if "EventTypes" in subscription:
            print( subscription_line_format.format( "", "Event Types", ", ".join(subscription["EventTypes"] ) ) )
        if "RegistryPrefixes" in subscription:
            print( subscription_line_format.format( "", "Registries", ", ".join( subscription["RegistryPrefixes"] ) ) )
        if "ResourceTypes" in subscription:
            print( subscription_line_format.format( "", "Resource Types", ", ".join( subscription["ResourceTypes"] ) ) )

def create_event_subscription( context, destination, format = None, client_context = None, expand = None, resource_types = None,
    registries = None, message_ids = None, origins = None, subordinate_resources = None, event_types = None ):
    """
    Creates an event subscription

    Args:
        context: The Redfish client object with an open session
        destination: The event subscription destination
        format: The format of the event payloads
        client_context: The client-provided context string
        expand: Indicates if the OriginOfCondition is to be expanded in event payloads
        resource_types: The resource types for the subscription
        registries: The registries for the subscription
        message_ids: The message IDs for the subscription
        origins: The origins for the subscription
        subordinate_resources: Indicates if subordinate resources to those referenced by 'origins' will also be monitored
        event_types: The event types for the subscription; this method for subscriptions has been deprecated for other controls

    Returns:
        The response of the POST
    """

    # Get the event service
    event_service = get_event_service( context )
    if "Subscriptions" not in event_service:
        # No subscriptions
        raise RedfishEventServiceNotFoundError( "Service does not contain a subscription collection" )

    # Form the POST request
    payload = {
        "Destination": destination,
        "Protocol": "Redfish"
    }
    if format is not None:
        payload["EventFormatType"] = format
    if client_context is not None:
        payload["Context"] = client_context
    if expand is not None:
        payload["IncludeOriginOfCondition"] = expand
    if resource_types is not None:
        payload["ResourceTypes"] = resource_types
    if registries is not None:
        payload["RegistryPrefixes"] = registries
    if message_ids is not None:
        payload["MessageIds"] = message_ids
    if origins is not None:
        payload["OriginResources"] = origins
    if subordinate_resources is not None:
        payload["SubordinateResources"] = subordinate_resources
    if event_types is not None:
        payload["EventTypes"] = event_types

    # Create the subscription
    response = context.post( event_service["Subscriptions"]["@odata.id"], body = payload )
    verify_response( response )
    return response

def delete_event_subscription( context, id ):
    """
    Deletes an event subscription

    Args:
        context: The Redfish client object with an open session
        id: The identifier for the subscription

    Returns:
        The response of the DELETE
    """

    # Get the current subscriptions
    subscriptions = get_event_subscriptions( context )

    # Find the matching subscription and delete it
    for subscription in subscriptions:
        if subscription["Id"] == id:
            response = context.delete( subscription["@odata.id"] )
            verify_response( response )
            return response

    # No matches found
    raise RedfishEventSubscriptionNotFoundError( "Service does not contain an event subscription called {}".format( id ) )



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2020 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/master/LICENSE.md

"""
Redfish System Inventory

File : rf_sys_inventory.py

Brief : This script uses the redfish_utilities module to dump system inventory
        information
"""

import argparse
import redfish
import redfish_utilities

# Get the input arguments
argget = argparse.ArgumentParser( description = "A tool to walk a Redfish service and list component information" )
argget.add_argument( "--user", "-u", type = str, required = True, help = "The user name for authentication" )
argget.add_argument( "--password", "-p",  type = str, required = True, help = "The password for authentication" )
argget.add_argument( "--rhost", "-r", type = str, required = True, help = "The address of the Redfish service (with scheme)" )
argget.add_argument( "--details", "-details", action = "store_true", help = "Indicates if the full details of each component should be shown" )
argget.add_argument( "--noabsent", "-noabsent", action = "store_true", help = "Indicates if absent devices should be skipped" )
argget.add_argument( "--write", "-w", nargs = "?", const = "Device_Inventory", type = str, help = "Indicates if the inventory should be written to a spreadsheet and what the file name should be if given" )
argget.add_argument( "--workaround", "-workaround", action = "store_true", help = "Indicates if workarounds should be attempted for non-conformant services", default = False )
args = argget.parse_args()

# Set up the Redfish object
redfish_obj = redfish.redfish_client( base_url = args.rhost, username = args.user, password = args.password )
redfish_obj.login( auth = "session" )

try:
    # Get and print the system inventory
    inventory = redfish_utilities.get_system_inventory( redfish_obj, args.workaround )
    redfish_utilities.print_system_inventory( inventory, args.details, args.noabsent )

    if args.write:
        redfish_utilities.write_system_inventory( inventory, args.write )

finally:
    # Log out
    redfish_obj.logout()
